import cx from 'classnames';
import { Overlay } from '../overlay/Overlay.jsx';
import style from './dialog.css';

function defaultRenderCloseButton () {
    return '\u2715';
}

function Dialog (
    {
        closeOnEsc = true,
        renderCloseButton = defaultRenderCloseButton,
        closeButtonClassName
    },
    children
) {
    function createHandlers (closeDialog) {
        function handleKeyDown ({which}) {
            (which === 27) && closeDialog();
        }

        return {
            onCreate () {
                document.addEventListener('keydown', handleKeyDown, true);
            },

            onRemove (element, done) {
                document.removeEventListener('keydown', handleKeyDown, true);
                done();
            }
        };
    }

    return function (state, actions) {
        if (!state.dialog.isOpen) {
            return null;
        }

        const closeDialog = actions.dialog.close;
        const elementHandlers = closeOnEsc ? createHandlers(closeDialog) : {};
        const {onCreate, onRemove} = elementHandlers;

        const composedCloseButtonClassName = cx(style.close, closeButtonClassName);

        return (
            <Overlay oncreate={onCreate} onremove={onRemove} onBackgroundClick={closeDialog}>
                <button className={composedCloseButtonClassName} onclick={closeDialog}>
                    {renderCloseButton()}
                </button>

                <div className={style.content}>
                    {children}
                </div>
            </Overlay>
        );
    };
}

export {
    Dialog
};
